<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Model\From\Entity;

/**
 * Class Product
 * @package FiloBlu\MappConnector\Model\From\Entity
 */
class Product extends \FiloBlu\MappConnector\Model\From\Entity\AbstractEntity implements \FiloBlu\MappConnector\Model\From\Entity\EntityInterface  {

    //TODO: Get catalog product per store

    /**
     * Entity
     *
     * @var string
     */
    protected $_entity = 'product';

    /**
     * Magento Entity Model
     *
     * @var string
     */
    protected $_entityModel = '\Magento\Catalog\Model\Product';

    /**
     * Magento Entity Model Id
     *
     * @var string
     */
    protected $_entityModelPrimary = 'entity_id';

    /**
     * Mapp Resource Model
     *
     * @var string
     */
    protected $_entityMappResource = '\FiloBlu\MappConnector\Model\To\Mapp\FTP';

    /**
     * Mapp Resource Model Id
     *
     * @var array
     */
    protected $_entityMappPrimary = ['primaryDB' => 'entity_id', 'primaryMapp' => 'entity_id'];


    /**
     * @var \Magento\Catalog\Model\Product
     */
    protected $_model;

    /**
     * @var string
     */
    protected $_xmlConfigPath = 'mappconnector/product_mapping/product';

    /**
     * @var \Magento\Eav\Api\AttributeSetRepositoryInterface
     */
    protected $_attributeSet;

    /**
     * @var \FiloBlu\MappConnector\Model\Config
     */
    protected $_config;

    /**
     * Product constructor.
     * @param \Magento\Catalog\Model\Product $product
     */
    public function __construct(
        \Magento\Catalog\Model\Product $product,
        \Magento\Eav\Api\AttributeSetRepositoryInterface $attributeSet,
        \FiloBlu\MappConnector\Model\Config $config
    )
    {
        $this->_model = $product;
        $this->_attributeSet = $attributeSet;
        $this->_config = $config;
    }

    /**
     * @return {@inheritDoc}
     */
    public function getObjectData($primary,$id)
    {
        return $this->_model->getCollection()
            ->addAttributeToSelect('*')
            ->addFieldToFilter($primary, $id)
            ->getFirstItem()
            ->getData();
    }

    /**
     * @return {@inheritDoc}
     */
    public function getDefaultData($record){
        $result = [];

        $result['Key'] = $record['sku'];

        //Get Attribute Set Name
        $attributeSetId = $record['attribute_set_id'];
        $attributeSetName = '';
        if(!empty($attributeSetId)) {
            $attributeSet = $this->_attributeSet->get($attributeSetId);
            $attributeSetName = $attributeSet->getAttributeSetName();
        }

        //$result['ProductURI'] = $product->getProductUrl();
        $result['AttributeSet'] = $attributeSetName;
        $result['CurrentPrice'] = $record['price'];
        $result['Name'] = $record['name'];

        return $result;
    }

    /**
     * @return mixed
     */
    public function getAllData($type = 'full', $last_index = '')
    {
        if($type == 'full') {
            //Full
            return $this->_model->getCollection()
            ->getData();
        }
        else {
            //Incremental
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance(); // Instance of object manager
            $resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
            $connection = $resource->getConnection();
            $tableName = $resource->getTableName('catalog_product_entity');
            $tableNameJobs = $resource->getTableName('filoblu_mappconnector_jobs');
            
            $sql = "SELECT 
                      ce.entity_id as entity_id
                    FROM
                      ".$tableName." AS ce
                    LEFT JOIN
                      ".$tableNameJobs." AS mj 
                    ON 
                      mj.entity_id = ce.entity_id
                    AND 
                      mj.entity = 'product'
                    WHERE 
                      mj.status is null";

            return $connection->fetchAll($sql);
        }
    }

}