<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Observer;

use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;

/**
 * Class CustomerRegister
 * @package FiloBlu\MappConnector\Observer
 */
abstract class AbstractObserver implements ObserverInterface
{

    /**
     * @var \FiloBlu\MappConnector\Model\Config
     */
    protected $_config;

    /**
     * AbstractObserver constructor.
     * @param \FiloBlu\MappConnector\Model\Jobs $jobs
     */
    public function __construct(
        \FiloBlu\MappConnector\Model\Jobs $jobs,
        \FiloBlu\MappConnector\Helper\FileLoggerHelper $fileLogger,
        \FiloBlu\MappConnector\Model\Config $config
    )
    {
        $this->_fileLogger = $fileLogger;
        $this->_jobs = $jobs;
        $this->_config = $config;
    }

    /**
     * @param Observer $observer
     * @return void
     */
    public function execute(Observer $observer)
    {
        //If Observer is enabled
        if($this->_config->getConfigValue('mappconnector/general/enable_observers')) {
            try {
                $this->_fileLogger->debug('Added new job for ' . $this->_entity . ' entity. ID: ' . $observer->getData($this->_type)->getId(),
                    [], __METHOD__ . ':' . __LINE__);
                $this->_jobs->insertJob([
                    'entity'    => $this->_entity,
                    'entity_id' => $observer->getData($this->_type)->getId(),
                    'mode'      => $this->_mode,
                    'status'    => $this->_status,
                    'type'      => 'incremental'
                ]);
            } catch (\Exception $e) {
                $this->_fileLogger->error('There was an error. The ' . $this->_entity . ' job has not been created (id -> ' . $observer->getData($this->_type)->getId() . '). Exception: ' . $e->getMessage());
            }
        }
    }
}